@@ -1,235 +1,227 @@
-#/*****************************************************************************
+/*****************************************************************************
  *                                                                           *
- * Programm:  paul                                                           *
+ * Program:   paul                                                           *
  *            (P)rogramm zur (A)uswertung und (U)mformung von                *
  *            (L)aserbildern                                                 *
  * Modul:     delbordr.c                                                     *
- *            Funktion zum Ausschneiden des eigentlichen Bildes aus einem    *
- *            Bild mit "dunklem" Rand                                        *
- * Autor:     Andreas Tille                                                  *
- * Datum:     27.02.1998                                                     *
+ *            Funktions to cut of the "dark" border of an image              *
+ * Author:    Andreas Tille                                                  *
+ * Date:      27.02.1998                                                     *
+ * Copyright: Andreas Tille, 1999; GNU Public License                        *
  *                                                                           *
  *****************************************************************************/
 
-#include <assert.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "paul.h" 
 
-#ifdef __DMALLOC__
-#include <dmalloc.h>
-#endif
-
 typedef void (*CountThreshFunc)(PICTURE *, int, int, int *, int *, int, int);
 
 static int TestLeftRight(int *lb, int *rb, int width, int pa, int pleft, int stp, int *found)
-/* ermittelt, ob left und ap "beiderseits" width liegen
+/* test, whether left and ap point to different rows
  * --- Parameter: ---
- * int           *lb    : Feld der linken Grenzen
- * int           *rb    : Feld der rechten Grenzen
- * int            width : Breite des Bildes
- * int            pa    : aktuelle Position
- * int            pleft : Begin des kritischen Randbereiches
- * int            stp   : Als 3 oder 1 Byte per Pixel gespeichert ?
- * int            found : Anzahl der bereits gefundenen Zeilen
- * --- R"uckgabe: ---
- * int           *lb    : neue linke Grenze eingetragen
- * int           *rb    : neue rechte Grenze eingetragen
- * int TestLeftRight()  : falls Randbereich erwischt, Offset bis zum 
- *                        voraussichtlich nchsten (halbe Bildbreite), sonst 0
+ * int           *lb    : field of left borders
+ * int           *rb    : field of right borders
+ * int            width : width of image
+ * int            pa    : current position
+ * int            pleft : beginn of critical (potential) border area
+ * int            stp   : store as 3 byte or 1 byte per pixel
+ * int            found : number of found valid rows
+ * --- Return: ---
+ * int           *lb    : field of left borders with new entry
+ * int           *rb    : field of right borders with new entry
+ * int TestLeftRight()  : if valid border area found assummed offset to a possible next 
+ *                        border (half of image width), else 0
  */
 {
-   register int i, j, storepix = stp;
+  register int i, j, storepix = stp;
 
-   if ( storepix != 1 ) {
-      pa    /= storepix;
-      pleft /= storepix;
-   }
-   if ( (pa / width) > (pleft / width) ) {
-      ++*found;
-      i = (*(rb + *found) = pleft % width) -
-          (*(lb + *found) = pa    % width);   
-      j = (i=MIN(i >> 1, width >> 2)) % storepix;
-      return i-j; /* sichern, da grnes Byte wieder getroffen wird! */
-   }
-   return 0;
+  if ( storepix != 1 ) {
+    pa    /= storepix;
+    pleft /= storepix;
+  }
+  if ( (pa / width) > (pleft / width) ) {
+    ++*found;
+    i = (*(rb + *found) = pleft % width) -
+        (*(lb + *found) = pa    % width);   
+    j = (i=MIN(i >> 1, width >> 2)) % storepix;
+    return i-j; /* make sure, that a green byte is used */
+  }
+  return 0;
 }
 
 static int WeightedMiddle(int *middle, int abw, int *data, int len)
-/* Ermittelt den Mittelwert von Daten, die hchstens um einen gewissen Betrag vom
- * vorberechneten Mittelwert abweichen
+/* calculate average of data, which doesn't differ more than a certain value
+ * from the precalculated average (excludes much to high or much to low values)
  * --- Parameter: ---
- * int *middle : vorgegebener Mittelwert
- * int  abw    : alle Werte mit abs(*middle - Wert) < abw werden gemittelt
- * int *data   : Werte
- * int  len    : Anzahl der Werte
- * --- R"uckgabe: ---
- * int *middle : neuer, "gewichteter" Mittelwert
- * int WeightedMiddle(): != 0, falls zu wenig Werte einbezogen werden konnten
- *                       ( < 25% )
+ * int *middle : precalculated average
+ * int  abw    : use only values with abs(*middle - value) < abw to calculate the weighted middle
+ * int *data   : data
+ * int  len    : number of data
+ * --- Return: ---
+ * int *middle : new, "weighted" average
+ * int WeightedMiddle(): != 0, if there where less than 25% of the data valid
  */
 {
-   register int *ap, orimiddle = *middle, oriabw = abw, sum = 0, nsum = 0, *fip;
+  register int *ap, orimiddle = *middle, oriabw = abw, sum = 0, nsum = 0, *fip;
    
-   for ( fip = (ap = data) + len; ap < fip; ap++ ) {
-      if ( abs(*ap - orimiddle) < oriabw ) {
-	 sum += *ap;
-         nsum++;
-      }
-   }
-   if ( nsum < (len >> 2) ) return nsum;
-   *middle = sum / nsum;
-   return 0;
+  for ( fip = (ap = data) + len; ap < fip; ap++ ) {
+    if ( abs(*ap - orimiddle) < oriabw ) {
+      sum += *ap;
+      nsum++;
+    }
+  }
+  if ( nsum < (len >> 2) ) return nsum;
+  *middle = sum / nsum;
+  return 0;
 }
 
 static void CountThreshCol(PICTURE *bild, int col, int testcols, 
                     int *nthresh, int *nterr, int thresh, int terr)
-/* Zhlt alle Werte der Umgebung einer Spalte die Stufe thresh und
- * Fehler terr berschreiten
+/* count all values of the environment of a column, which are greater than thresh and terr
  * --- Parameter: ---
- * PICTURE *bild       : Bild
- * int      col        : zu testende Spalte
- * int      testcols   : Anzahl der zu untersuchenden Spalten
- * int     *nthresh    : Feld fr Schwellwertauftreten
- * int     *nterr      : Feld fr Fehlerauftreten
- * unsigned char thresh: Schwellwert fr "dunkel"
- * unsigned char terr  : Toleranz fr einzelne Werte
- * --- Rueckgabe: ---
- * int     *nthresh    : Feld mit Schwellwertauftreten
- * int     *nterr      : Feld mit Fehlerauftreten
- * int CountThreshCol(): neue Spalte, im Fehlerfall -1
+ * PICTURE *bild       : image
+ * int      col        : column to test
+ * int      testcols   : number of columns to test
+ * int     *nthresh    : field to count number of values greater than thresh for each column
+ * int     *nterr      : field to count number of values greater than terr for each column
+ * unsigned char thresh: threshold to call a value "dark"
+ * unsigned char terr  : allowed tolerance for a single pixel
+ * --- Return: ---
+ * int     *nthresh    : field to count number of values greater than thresh for each column
+ * int     *nterr      : field to count number of values greater than terr for each column
+ * int CountThreshCol(): new column, -1 in case of error
  */
 {
-   register int *ip, *jp, *fip, storepix = bild->storepix;
-   register unsigned char *ap, *fap;
-   int      step = bild->W;
+  register int *ip, *jp, *fip, storepix = bild->storepix;
+  register unsigned char *ap, *fap;
+  int      step = bild->W;
 
-   step = storepix*(step - testcols); 
-        /* bild->storepix*testcols Byte werden in innerer Schleife behandelt! */
+  step = storepix*(step - testcols); 
+       /* bild->storepix*testcols byte will be handled in inner loop! */
    
-   for ( fip = (ip = nthresh) + testcols, jp = nterr; 
-         ip < fip; ip++, jp++ ) 
-      *ip = *jp = 0;
+  for ( fip = (ip = nthresh) + testcols, jp = nterr; 
+        ip < fip; ip++, jp++ ) 
+    *ip = *jp = 0;
 
-   for ( fap = bild->DATA + storepix*bild->size, 
-         ap  = bild->DATA + storepix*(col - (testcols>>1)) + (storepix == 3 ? 1 : 0);  
-                                                             /* +1 wegen GRN!! */
-         ap < fap; ap += step ) {
-      for ( fip = (ip = nthresh) + testcols, jp = nterr; 
-            ip < fip; ip++, jp++, ap += storepix ) {         /* +3 wegen GRN!! */
-	 if ( *ap > thresh ) {
-            ++*ip;
-            if ( *ap > terr ) ++*jp;
-	 }
+  for ( fap = bild->DATA + storepix*bild->size, 
+        ap  = bild->DATA + storepix*(col - (testcols>>1)) + (storepix == 3 ? 1 : 0);  
+                                                             /* +1 take GREEN!! */
+        ap < fap; ap += step ) {
+    for ( fip = (ip = nthresh) + testcols, jp = nterr; 
+          ip < fip; ip++, jp++, ap += storepix ) {         /* +3 take GREEN!! */
+      if ( *ap > thresh ) {
+        ++*ip;
+        if ( *ap > terr ) ++*jp;
       }
-   }
+    }
+  }
 }
 
 static void CountThreshRow(PICTURE *bild, int row, int testrows, 
                     int *nthresh, int *nterr, int thresh, int terr)
-/* Zhlt alle Werte der Umgebung einer Zeile die Stufe thresh und
- * Fehler terr berschreiten
+/* count all values of the environment of a row, which are greater than thresh and terr
  * --- Parameter: ---
- * PICTURE *bild       : Bild
- * int      row        : zu testende Zeile
- * int      testrows   : Anzahl der zu untersuchenden Zeilen
- * int     *nthresh    : Feld fr Schwellwertauftreten
- * int     *nterr      : Feld fr Fehlerauftreten
- * unsigned char thresh: Schwellwert fr "dunkel"
- * unsigned char terr  : Toleranz fr einzelne Werte
- * --- Rueckgabe: ---
- * int     *nthresh    : Feld mit Schwellwertauftreten
- * int     *nterr      : Feld mit Fehlerauftreten
- * int CountThreshRow(): neue Zeile, im Fehlerfall -1
+ * PICTURE *bild       : image
+ * int      row        : row to test
+ * int      testrows   : number of rows to test
+ * int     *nthresh    : field to count number of values greater than thresh for each row
+ * int     *nterr      : field to count number of values greater than terr for each row
+ * unsigned char thresh: threshold to call a value "dark"
+ * unsigned char terr  : allowed tolerance for a single pixel
+ * --- Return: ---
+ * int     *nthresh    : field to count number of values greater than thresh for each row
+ * int     *nterr      : field to count number of values greater than terr for each row
+ * int CountThreshRow(): new column, -1 in case of error
  */
 {
-   register int           *ip, *jp, storepix = bild->storepix;
-   register unsigned char *bp, *fbp, *ap, *fap;
-   int                    *fip;
+  register int           *ip, *jp, storepix = bild->storepix;
+  register unsigned char *bp, *fbp, *ap, *fap;
+  int                    *fip;
 
-   for ( fip = (ip = nthresh) + testrows, jp = nterr; 
-         ip < fip; ip++, jp++ ) 
-      *ip = *jp = 0;
+  for ( fip = (ip = nthresh) + testrows, jp = nterr; 
+        ip < fip; ip++, jp++ ) 
+    *ip = *jp = 0;
 
-   for ( ip = nthresh, jp = nterr, ap = bild->DATA + storepix*(row - (testrows>>1)),
-         fap = bild->DATA + storepix*bild->size; 
-         ip < fip; ip++, jp++ ) 
+  for ( ip = nthresh, jp = nterr, ap = bild->DATA + storepix*(row - (testrows>>1)),
+        fap = bild->DATA + storepix*bild->size; 
+        ip < fip; ip++, jp++ ) 
 
-   ap = bild->DATA + storepix*(row - (testrows>>1)) + (storepix == 3 ? 1 : 0); /*+1 wegen GRN!!*/
-   for ( fap = ap + storepix*testrows*bild->W, ip = nthresh, jp = nterr; 
-         ap < fap; ap = fbp, ip++, jp++ ) {
-      for ( fbp = (bp = ap) + storepix * bild->W; bp < fbp; bp += storepix ) { /*+3 wegen GRN!!*/
-	 if ( *bp > thresh ) {
-            ++*ip;
-            if ( *bp > terr ) ++*jp;
-	 }
+  ap = bild->DATA + storepix*(row - (testrows>>1)) + (storepix == 3 ? 1 : 0); /*+1 take GREEN!!*/
+  for ( fap = ap + storepix*testrows*bild->W, ip = nthresh, jp = nterr; 
+        ap < fap; ap = fbp, ip++, jp++ ) {
+    for ( fbp = (bp = ap) + storepix * bild->W; bp < fbp; bp += storepix ) { /*+3 take GREEN!!*/
+      if ( *bp > thresh ) {
+        ++*ip;
+        if ( *bp > terr ) ++*jp;
       }
-   }
+    }
+  }
 }
 
 static int CheckBorderColumn(PICTURE *bild, int *col, unsigned char thresh, 
                              unsigned char terr, int dir, int flag)
-/* Testet die Umgebung einer Bezugsspalte auf "Dunkelheit"
- * FALLS bild->storepix == 3 WIRD NUR GRN GETESTET!!!!!!!!!!!!!!!!!!!
+/* test environment of a certain column for "darkness"
+ * IF bild->storepix == 3 ONLY GREEN (Byte 2) WILL BE TESTET!!!!!!!!!!!!!!!!!!!
  * --- Parameter: ---
- * PICTURE *bild          : Bild
- * int     *col           : zu testende Spalte
- * unsigned char thresh   : Schwellwert fr "dunkel"
- * unsigned char terr     : Toleranz fr einzelne Werte
- * int      dir           : Richtung, in der Rand zu suchen ist 
- *                          (wenn *col = *left ist dir = -1)
- * int      flag          : wie oft wurde CheckBorderColumn() schon iterativ
- *                          aufgerufen
- * --- Rueckgabe: ---
- * int     *col           : gegebenenfalls neue Spalte, die Rand besser trifft
- * int CheckBorderColumn(): neue Spalte, im Fehlerfall -1
+ * PICTURE *bild          : image
+ * int     *col           : column to test
+ * unsigned char thresh   : threshold to call a value "dark"
+ * unsigned char terr     : allowed tolerance for a single pixel
+ * int      dir           : direction, where to search for the border 
+ *                          (if *col = *left then dir = -1)
+ * int      flag          : how often was CheckBorderColumn() called recursively
+ * --- Return: ---
+ * int     *col           : new column, if it matches border better
+ * int CheckBorderColumn(): new column, -1 in case of an error
  */
 {
-#define TESTCOLS 11        /* wieviel Spalten auf einmal testen? */
-#define NITER    10        /* wie tief rekursive Aufrufe?        */
-   int                    nthresh[TESTCOLS], nterr[TESTCOLS], 
-                          tolthr, tolerr, neucol;
-   register int           *ip, *jp, *fip, i;
+#define TESTCOLS 11        /* how many columns to test in one step? */
+#define NITER    10        /* how many recursive iterations         */
+  int                    nthresh[TESTCOLS], nterr[TESTCOLS], 
+                         tolthr, tolerr, neucol;
+  register int           *ip, *jp, *fip, i;
 
-   if ( flag > NITER ) return -1;
-      /* Nach mehr als NITER Iterationen konnte kein sinnvoller Rand gefunden werden! */
+  if ( flag > NITER ) return -1;
+    /* No valid border after more than NITER recursive iterations! */
 
-   if ( *col <  TESTCOLS>>1           ) *col = TESTCOLS>>1;             /* linken Rand exakt erwischen  */
-   if ( *col >= bild->W-(TESTCOLS>>1) ) *col = bild->W-(TESTCOLS>>1)-1; /* rechten Rand exakt erwischen */
-   CountThreshCol(bild, *col, TESTCOLS, nthresh, nterr, thresh, terr);
+  if ( *col <  TESTCOLS>>1           ) *col = TESTCOLS>>1;             /* exact match of left border */
+  if ( *col >= bild->W-(TESTCOLS>>1) ) *col = bild->W-(TESTCOLS>>1)-1; /* exact match of right border */
+  CountThreshCol(bild, *col, TESTCOLS, nthresh, nterr, thresh, terr);
 
-   /*   tolthr = bild->W >> 5;   1/32 darf drber liegen     */
-   /*   tolerr = tolthr >> 3;    1/256 darf ber Fehler sein */
-   tolthr = bild->W >> 4;  /* 1/16 darf drber liegen     */
-   tolerr = tolthr >> 3;   /* 1/128 darf ber Fehler sein */
-   if ( tolthr < 4 ) tolthr = 4;
-   if ( tolerr < 1 ) tolerr = 1;
-   /* ... damit es bei kleinen Bildern auch klappt!! */
+  /*   tolthr = bild->W >> 5;   1/32 of data is allowed to fail the threshold test           */
+  /*   tolerr = tolthr >> 3;    1/256 of data is allowed to fail the single pixel error test */
+  tolthr = bild->W >> 4;  /* 1/16  of data is allowed to fail the threshold test          */
+  tolerr = tolthr >> 3;   /* 1/128 of data is allowed to fail the single pixel error test */
+  if ( tolthr < 4 ) tolthr = 4;
+  if ( tolerr < 1 ) tolerr = 1;
+  /* ... useful and sometimes necessary for small images!! */
     
-   neucol = -1;
-   for ( ip = nthresh, jp = nterr, 
-         fip = nthresh + TESTCOLS, i = *col - (TESTCOLS>>1);
-         ip < fip; ip++, jp++, i++ ) {
-      if ( *ip < tolthr && *jp < tolerr ) {
-         if ( dir > 0 ) {
-            if ( ip == nthresh )  /* schon am Anfang nicht im Bild => gehe nach links */
-               i = (neucol = TESTCOLS) + 1;
-            else
-               neucol = i;
-            break;
-	 } else {  /* linker Rand */
-            neucol = i;
-	 }
+  neucol = -1;
+  for ( ip = nthresh, jp = nterr, 
+        fip = nthresh + TESTCOLS, i = *col - (TESTCOLS>>1);
+        ip < fip; ip++, jp++, i++ ) {
+    if ( *ip < tolthr && *jp < tolerr ) {
+      if ( dir > 0 ) {
+        if ( ip == nthresh )  /* schon am Anfang nicht im Bild => gehe nach links */
+          i = (neucol = TESTCOLS) + 1;
+        else
+          neucol = i;
+        break;
+      } else {  /* linker Rand */
+        neucol = i;
       }
-   }
+    }
+  }
 
-   if ( neucol == --i || neucol < 0 ) {
-      neucol = (neucol < 0 ? *col + dir*(TESTCOLS-1) : *col - dir*(TESTCOLS-1));
-      if ( CheckBorderColumn(bild, &neucol, thresh, terr, dir, ++flag) < 0 ) return -1;
-   }
+  if ( neucol == --i || neucol < 0 ) {
+    neucol = (neucol < 0 ? *col + dir*(TESTCOLS-1) : *col - dir*(TESTCOLS-1));
+    if ( CheckBorderColumn(bild, &neucol, thresh, terr, dir, ++flag) < 0 ) return -1;
+  }
 
-   return *col = neucol;
+  return *col = neucol;
 #undef TESTCOLS
 #undef NITER
 }
@@ -247,7 +239,7 @@ static int NewBorder(PICTURE *bild, int oldwd, int neuwd, int *left, int *right,
  * unsigned char   thresh      : Schwellwert fr "dunkel"
  * unsigned char   terr        : Toleranz fr einzelne Werte
  * CountThreshFunc *ctf        : Funktion zur Berechnung der Anzahl von Fehlern in Spalten/Zeilen
- * --- Rueckgabe: ---
+ * --- Return: ---
  * int             *left       : neue linke/obere Grenze
  * int             *right      : neue rechte/untere Grenze
  * int             NewBorder() : 0 fr OK
@@ -265,7 +257,7 @@ static int NewBorder(PICTURE *bild, int oldwd, int neuwd, int *left, int *right,
    if ( (delta & 1) == 0 ) delta++;  /* ungeradzahlig geht besser! */
 
    /* delta+2, weil links und rechts von delta gesucht wird!!!! */
-   assert ( (nthl = calloc((delta+2)<<2, sizeof(int))) != NULL );
+   nthl = g_new0(int, (delta+2) << 2 );
    nter = (nthr = (ntel = nthl + delta) + delta) + delta;
    
    if ( (zw = *left - (delta/2) ) < 0 ) {
@@ -340,7 +332,7 @@ static int NewBorder(PICTURE *bild, int oldwd, int neuwd, int *left, int *right,
 	 }
       }
    } while ( *right - *left != neuwd );
-   free(nthl);
+   FREE(nthl);
    return 0;
 
 #undef NEXTCOL   
@@ -360,23 +352,23 @@ static int GetBorder(PICTURE *bild, int *left, int *right, unsigned char thresh,
  * int      *right      : Zeiger auf rechte Grenze (erste Spalte auerhalb des Bildes)
  * unsigned char thresh : Schwellwert fr "dunkel"
  * unsigned char terr   : Toleranz fr einzelne Werte
- * --- Rueckgabe: ---
+ * --- Return: ---
  * int      *left       : linke Grenze
  * int      *right      : rechte Grenze
- * int      GetBorder() : 0 fr OK
+ * int      GetBorder() : RET_OK or RET_OK
  */
 {
    register unsigned char *ap, *fip;
    register int flag = 0, bad = 0, *lbp, *rbp, storepix;
    unsigned char *tleft, *start = bild->DATA;
    int      width = bild->W, *lb, *rb, makesense, dobad, foundlines = 0, *fop;
-   char     *nonsense = "Es kann kein sinnvoller Bildrand mit diesen Parametern gefunden werden (%s).\n";
+   char     *nonsense = _("No valid image borders can be found using thresh = %i and terr = %i (%s).");
 
-   storepix = bild->storepix;
-   *left    = *right = 0;
-   tleft    = start;
-   assert ( (lb = calloc(2*bild->H, sizeof(int))) != NULL );
-   rb = lb + (makesense = bild->H);
+   storepix   = bild->storepix;
+   *left      = *right = 0;
+   tleft      = start;
+   lb         = g_new0(int, bild->H << 1);
+   rb         = lb + (makesense = bild->H);
    makesense *= 100;
    
    if ( storepix == 3 ) 
@@ -424,9 +416,9 @@ static int GetBorder(PICTURE *bild, int *left, int *right, unsigned char thresh,
       }
    }
    if ( !makesense ) {
-      fprintf(stderr, nonsense, "erster Scan fehlgeschlagen");
-      free(lb);
-      return -1;
+      g_warning(nonsense, thresh, terr, _("first scan failed"));
+      FREE(lb);
+      return RET_ERR;
    }
    if ( bild->H < 3*foundlines ) {
       flag = bad = 0;  /* Registervariablen ausnutzen zum Summen speichern. */   
@@ -437,9 +429,9 @@ static int GetBorder(PICTURE *bild, int *left, int *right, unsigned char thresh,
       *left  = flag / foundlines;
       *right = bad  / foundlines;
       if ( !*right || (!*left && *right == width) ) {
-         fprintf(stderr, nonsense, "Rand unsinnig");
-         free(lb);
-         return -1;
+         g_warning(nonsense, thresh, terr, _("invalid border"));
+         FREE(lb);
+         return RET_ERR;
       }
 
       /* Noch einmal Durchschnitt ermitteln, aber Ausreier herauslassen! */
@@ -461,9 +453,9 @@ static int GetBorder(PICTURE *bild, int *left, int *right, unsigned char thresh,
          *left  = 5;
 	 *right = width - 5;
       } else {
-         fprintf(stderr, nonsense, "zu wenig gltige Zeilen");
-         free(lb);
-         return -1;
+         g_warning(nonsense, thresh, terr, _("to less valid rows"));
+         FREE(lb);
+         return RET_ERR;
       }
    }
    
@@ -475,14 +467,14 @@ static int GetBorder(PICTURE *bild, int *left, int *right, unsigned char thresh,
       return -1;
 
    if ( *left >= *right ) {
-      fprintf(stderr, nonsense, "senkrechter Feinscan");
-      free(lb);
+      g_warning(nonsense, thresh, terr, _("vertical control scan"));
+      FREE(lb);
       *left = flag; *right = bad;
-      return -1;
+      return RET_ERR;
    }
    ++*left;    /* Linker Rand soll innerhalb des Bildes sein!! */
-   free(lb);
-   return 0;
+   FREE(lb);
+   return RET_OK;
 }
 
 int CreateCheckLine(PICTURE *bild, int *lb, int *rb)
@@ -492,7 +484,7 @@ int CreateCheckLine(PICTURE *bild, int *lb, int *rb)
  * PICTURE *bild        : Bild
  * int      *lb         : Zeiger auf linke Grenze
  * int      *rb         : Zeiger auf rechte Grenze
- * --- Rueckgabe: ---
+ * --- Return: ---
  * int      *lb         : linke Grenze
  * int      *rb         : rechte Grenze
  * int CreateCheckLine(): neue Breite
@@ -534,10 +526,10 @@ static int GetTopBottom(PICTURE *bild, int *top, int *bot, unsigned char thresh,
  * int      *bot           : Zeiger auf untere Grenze (erste Zeile auerhalb des Bildes)
  * unsigned char thresh    : Schwellwert fr "dunkel"
  * unsigned char terr      : Toleranz fr einzelne Werte
- * --- Rueckgabe: ---
+ * --- Return: ---
  * int      *top           : obere Grenze
  * int      *bot           : untere Grenze
- * int      GetTopBottom() : 0 fr OK
+ * int      GetTopBottom() : RET_OK or RET_ERR
  */
 {
    register unsigned char *bp, *fbp;
@@ -599,8 +591,8 @@ static int GetTopBottom(PICTURE *bild, int *top, int *bot, unsigned char thresh,
       if ( ip > tolthr || jp > tolerr ) break;
    }
    *bot = ((ap-bild->DATA+2)/storepix)/bild->W;
-   if ( *top == 0 && *bot == bild->H ) return -1;
-   return 0;
+   if ( *top == 0 && *bot == bild->H ) return RET_ERR;
+   return RET_OK;
 }
 
 int SelectPictureWithoutBorder(PAUL *p)
@@ -615,133 +607,116 @@ int SelectPictureWithoutBorder(PAUL *p)
  *                                     thresh : threshold for dark pixels
  *                                     terr   : threshold for single (noisy) pixels
  *                                     flag   : used, if only left and right border should be cut
- * --- R"uckgabe: ---
- * int   SelectPictureWithoutBorder(): if OK 0, else -1
+ * --- return: ---
+ * int   SelectPictureWithoutBorder(): RET_OK or RET_ERR
  */
 {
-   PICTURE        *bild;
-   register GList *pl, *piclist;
-   int            *lb, *rb, *wd, *testlr, *tb, *bb, *testtb, i, h, neuwd, neuht = 0;
-   register       unsigned char *ap, *np;
-   unsigned       char *buf, *fap;
-   char           *msg = "%s remains unchanged.\n", desc[512];
+  PICTURE        *bild;
+  register GList *pl, *piclist;
+  int            *lb, *rb, *wd, *testlr, *tb, *bb, *testtb, i, h, neuwd, neuht = 0;
+  char           *lrdesc, *tbdesc, *desc;
    
-   piclist = p->piclist;
-   if ( !NBILDER(piclist) ) return 0;
+  g_return_val_if_fail ( IS_PAUL(p), RET_ERR );
+  g_return_val_if_fail ( CheckPicList(p->piclist), RET_ERR );
+  piclist = p->piclist;
 
-   assert ( (lb = calloc(7*(i = NBILDER(piclist)), sizeof(int))) != NULL );
-   testtb = (testlr = ( wd = ( bb = ( tb = (rb = lb + i)+i)+i)+i)+i)+i;
+  lb     = g_new0(int, 7*(i = NBILDER(piclist)));
+  testtb = (testlr = ( wd = ( bb = ( tb = (rb = lb + i)+i)+i)+i)+i)+i;
 
-   /* seitlichen Rand abschneiden */   
-   h = neuwd = 0;
+  /* Crop left and right border */   
+  h = neuwd = 0;
 
-   for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
-      if ( (*(testlr+i) = 
-            GetBorder(bild, lb+i, rb+i, p->opt->thresh, p->opt->terr)) == 0 ) {
-         neuwd += (*(wd+i) = *(rb+i) - *(lb+i));
-         h++;
-      }
-   }
-   if ( h > 1 )   /* Durchschnitt lieber etwas mehr */
-      neuwd = (int)( neuwd + ((3*h)>>2) ) / h;   
-   for ( bild = BILD(pl = piclist); pl; bild = BILD(pl = pl->next)) 
-      neuwd = MIN(neuwd, bild->W);
+  for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
+    if ( (*(testlr+i) = GetBorder(bild, lb+i, rb+i, p->opt->thresh, p->opt->terr)) == RET_OK ) {
+      neuwd += (*(wd+i) = *(rb+i) - *(lb+i));
+      h++;
+    }
+  }
+  if ( h > 1 )   /* let the average be a little bit more if unsure */
+    neuwd = (int)( neuwd + ((3*h)>>2) ) / h;   
+  for ( bild = BILD(pl = piclist); pl; bild = BILD(pl = pl->next)) 
+    neuwd = MIN(neuwd, bild->W);
    
-   for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
-      if ( *(testlr+i) ) continue;
-      if ( *(wd+i) != neuwd ) {
-	 if ( NewBorder(bild, bild->W, neuwd, lb+i, rb+i, p->opt->thresh, p->opt->terr, CountThreshCol) ) {
-            fprintf(stderr, "Width of %s cant be changed to %i.\n", bild->file, neuwd);
-	    continue; 
-	 }
+  for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
+    /* delete border left and right */
+    if ( *(testlr+i) ) continue;
+    if ( *(wd+i) != neuwd ) {
+      if ( NewBorder(bild, bild->W, neuwd, lb+i, rb+i, p->opt->thresh, p->opt->terr, CountThreshCol) ) {
+        g_warning(_("Width of %s can't be changed to %i."), bild->file, neuwd);
+        continue; 
       }
+    }
 
-      *(wd+i) = neuwd;  /* OHNE CHECKLINE!!!! */
-      /*      *(wd+i) = CreateCheckLine(bild, lb+i, rb+i);  */
+    *(wd+i) = neuwd;  /* WITHOUT CHECKLINE!!!! */
+    /*      *(wd+i) = CreateCheckLine(bild, lb+i, rb+i);  */
 
-      assert ( (buf = malloc(bild->storepix * *(wd+i) * bild->H)) != NULL );
-      for ( fap = (ap = bild->DATA + bild->storepix * *(lb+i)) + bild->storepix*bild->size,
-            np  = buf;
-            ap < fap; 
-            ap += bild->storepix * bild->W,
-            np += bild->storepix * *(wd+i) ) {
-         memcpy(np, ap, bild->storepix * *(wd+i));
+    CropImage(bild, *(lb+i), 0, *(wd+i), bild->H);
+  }
+
+  if ( !OnlyLeftRight(p->opt->f) ) {
+    /* Crop top and bottom border */
+    h = neuht = 0;
+    for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
+      if ( (*(testtb+i) = GetTopBottom(bild, tb+i, bb+i, p->opt->thresh, p->opt->terr)) == RET_OK ) {
+        neuht += (*(wd+i) = *(bb+i) - *(tb+i));
+        h++;
       }
-      NewImage(bild, *(wd+i), bild->H, buf);
-      bild->size = bild->W * bild->H;
-   }
+    }
+    if ( h > 1 )   /* Average and a little bit more ... */
+      neuht = (int)( neuht + ((3*h)>>2) ) / h;   
+    for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
+      neuht = MIN(neuht, bild->H);
+      if ( *(testtb+i) && (neuht < bild->H) && NBILDER(piclist) > 1 ) {
+        g_warning(_("No change of height for all images because %s can't be cut vertically."), bild->file);
+        neuht = 0;
+        break;
+      }
+    }
 
-   if ( !OnlyLeftRight(p->opt->f) ) {
-      /* Rand oben und unten abschneiden */
-      h = neuht = 0;
+    if ( neuht ) {
       for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
-            if ( (*(testtb+i) = 
-               GetTopBottom(bild, tb+i, bb+i, p->opt->thresh, p->opt->terr)) == 0 ) {
-            neuht += (*(wd+i) = *(bb+i) - *(tb+i));
-            h++;
-         }
-      }
-      if ( h > 1 )   /* Durchschnitt lieber etwas mehr */
-         neuht = (int)( neuht + ((3*h)>>2) ) / h;   
-      for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
-         neuht = MIN(neuht, bild->H);
-         if ( *(testtb+i) && (neuht < bild->H) && NBILDER(piclist) > 1 ) {
-            fprintf(stderr, "No change of height for all images because %s can't be cut vertically.\n",
-                    bild->file);
-            neuht = 0;
-            break;
-	 }
-      }
+        if ( *(wd+i) != neuht ) {
+          if ( NewBorder(bild, bild->H, neuht, tb+i, bb+i, p->opt->thresh, p->opt->terr, CountThreshRow) ) {
+            g_warning(_("Height of %s can't be changed to %i."), bild->file, neuht);
+            continue; 
+          }
+        }
+        *(wd+i) = neuht;  /* WITHOUT CHECKLINE!!!! */
+        /*      *(wd+i) = CreateVerticalCheckLine(bild, tb+i, bb+i);  */
 
-      if ( neuht ) {
-         for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
-            if ( *(wd+i) != neuht ) {
-               if ( NewBorder(bild, bild->H, neuht, tb+i, bb+i, p->opt->thresh, p->opt->terr, CountThreshRow) ) {
-                  fprintf(stderr, "Height of %s cant be changed to %i.\n", 
-                          bild->file, neuht);
-                  continue; 
-               }
-            }
-            *(wd+i) = neuht;  /* OHNE CHECKLINE!!!! */
-            /*      *(wd+i) = CreateVerticalCheckLine(bild, tb+i, bb+i);  */
-            assert ( (buf = malloc(bild->storepix * *(wd+i) * bild->W)) != NULL );
-            bild->size = ( bild->H = *(wd+i) ) * bild->W;
-            memcpy(buf, bild->DATA + *(tb+i)*bild->W*bild->storepix, bild->storepix*bild->size);
-            NewImage(bild, bild->W, *(wd+i), buf);
-            bild->size = bild->W * bild->H;
-	 }
+        CropImage(bild, 0, *(tb+i), bild->W, *(wd+i));
       }
-   }
+    }
+  }
 
-   /* set spezification chunks in the end */
-   for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
-      if ( *(testlr+i) && !neuht ) {   /* unterschiedliche Testbedingungen ... mal sehn, ob das gut ist*/
-         printf(msg, bild->file);
-	 continue;
-      }
-      if ( DelScanBorder(p->opt->f) ) {
-         NewFileName(bild, APPDELSCANBORDER);
-         CopySpec(bild->spec, ChunkNameTyp, TypDelScanRand);
-         strcpy(desc, "Scanner border (white) deleted:");
-         p->opt->f  &= ~DELSCANBORDER;
-	 bild->flag |= DELSCANBORDER;
-      } else {
-         char typ[256];
-	 
-         sprintf(typ, "%s (+%i-%i)", TypDelRand, p->opt->thresh, p->opt->terr);
-         NewFileName(bild, APPDELBORDER);
-         CopySpec(bild->spec, ChunkNameTyp, typ);
-         strcpy(desc, "New border:");
-         p->opt->f  &= ~DELBORDER;
-	 bild->flag |= DELBORDER;
-      }
-      if ( !*(testlr+i) )
-         sprintf(desc + strlen(desc), " (left = %i, right = %i)", *(lb+i), *(rb+i));
-      if ( !*(testtb+i) )
-         sprintf(desc + strlen(desc), " (top = %i, bottom = %i)", *(tb+i), *(bb+i));
-      CreateDescription(bild, desc);
-   }
-   free(lb);
-   return 0;
+  /* set spezification chunks in the end */
+  for ( bild = BILD(pl = piclist), i = 0; pl; bild = BILD(pl = pl->next), i++ ) {
+    if ( *(testlr+i) && !neuht ) {   /* unterschiedliche Testbedingungen ... mal sehn, ob das gut ist*/
+      g_warning(_("%s remains unchanged."), bild->file);
+      continue;
+    }
+    if ( !*(testlr+i) ) lrdesc = g_strdup_printf("(left = %i, right = %i)", *(lb+i), *(rb+i));
+    else                lrdesc = g_strdup("");
+    if ( !*(testtb+i) ) tbdesc = g_strdup_printf("(top = %i, bottom = %i)", *(tb+i), *(bb+i));
+    else                tbdesc = g_strdup("");
+
+    if ( DelScanBorder(p->opt->f) ) {
+      desc = g_strdup_printf("White scanner border of %s deleted: %s %s", 
+                              ImgFileName(bild), lrdesc, tbdesc);
+      ImgChunksUpdate(bild, TypDelScanRand, desc, APPDELSCANBORDER, DELSCANBORDER);
+      p->opt->f  &= ~DELSCANBORDER;
+    } else {
+      desc = g_strdup_printf("New border of %s: (+%i-%i): %s %s", 
+                            ImgFileName(bild), p->opt->thresh, p->opt->terr, lrdesc, tbdesc);
+      ImgChunksUpdate(bild, TypDelRand, desc, APPDELBORDER, DELBORDER);
+      p->opt->f  &= ~DELBORDER;
+    }
+    FREE(lrdesc);
+    FREE(tbdesc);
+    FREE(desc);
+  }
+
+  FREE(lb);
+  return RET_OK;
 }
 
